% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expected.item.R
\name{expected.item}
\alias{expected.item}
\title{Function to calculate expected value of item}
\usage{
expected.item(x, Theta, min = 0)
}
\arguments{
\item{x}{an extracted internal mirt object containing item information (see \code{\link{extract.item}})}

\item{Theta}{a vector (unidimensional) or matrix (multidimensional) of latent trait values}

\item{min}{a constant value added to the expected values indicating the lowest theoretical
category. Default is 0}
}
\description{
Given an internal mirt object extracted from an estimated model
compute the expected value for an item given the ability parameter(s).
}
\examples{

mod <- mirt(Science, 1)
extr.2 <- extract.item(mod, 2)
Theta <- matrix(seq(-6,6, length.out=200))
expected <- expected.item(extr.2, Theta, min(Science[,1])) #min() of first item
head(data.frame(expected, Theta=Theta))

}
\references{
Chalmers, R., P. (2012). mirt: A Multidimensional Item Response Theory
Package for the R Environment. \emph{Journal of Statistical Software, 48}(6), 1-29.
\doi{10.18637/jss.v048.i06}
}
\seealso{
\code{\link{extract.item}}, \code{\link{expected.test}}
}
\author{
Phil Chalmers \email{rphilip.chalmers@gmail.com}
}
\keyword{expected}
\keyword{value}
